/**
 * @file	cf_queue.c
 * @author	myusgun <myusgun@gmail.com>
 */
#include "cf_queue.h"
#include "cf_list.h"
#include "cf_error.h"

#include <stdio.h>

/**
 * 큐 컨텍스트 생성
 *
 * @return 성공 시, CF_OK; 실패 시, 오류 코드
 *
 * @param ctx 큐 컨텍스트 포인터
 */
int
CF_Queue_CreateCtx (CF_Queue_Ctx * ctx)
{
	return CF_List_CreateCtx ((CF_List_Ctx *) ctx);
}

/**
 * 큐 컨텍스트 해제
 *
 * @return 성공 시, CF_OK; 실패 시, 오류 코드
 *
 * @param ctx 큐 컨텍스트
 */
int
CF_Queue_DestroyCtx (CF_Queue_Ctx ctx)
{
	return CF_List_DestroyCtx (ctx);
}

/**
 * 큐에 삽입
 *
 * @return 성공 시, CF_OK; 실패 시, 오류 코드
 *
 * @param ctx		큐 컨텍스트
 * @param element	데이터 주소
 */
int
CF_Queue_Put (CF_Queue_Ctx	ctx,
			  const void	* element)
{
	int				result = 0;
	CF_List_Ctx		list = (CF_List_Ctx) ctx;
	CF_Traverser	traverser = NULL;

	result = CF_List_Rear (list, &traverser);
	if (result < 0)
		return result;

	return CF_List_Insert (list, traverser, CF_DIRECTION_AFTER, element);
}

/**
 * 큐에서 꺼내기
 *
 * @return 성공 시, CF_OK; 실패 시, 오류 코드
 *
 * @param ctx		큐 컨텍스트
 * @param element	데이터 주소
 */
int
CF_Queue_Get (CF_Queue_Ctx	ctx,
			  void			** element)
{
	int			result = 0;
	CF_List_Ctx	list = (CF_List_Ctx) ctx;
	CF_Traverser	traverser = NULL;

	result = CF_Queue_Front (ctx, element);
	if (result < 0)
		return result;

	result = CF_List_Front (list, &traverser);
	if (result < 0)
		return result;

	return CF_List_Remove (ctx, &traverser);
}

/**
 * 큐 처음의 데이터 조회
 *
 * @return 성공 시, CF_OK; 실패 시, 오류 코드
 *
 * @param ctx		큐 컨텍스트
 * @param element	데이터 주소
 */
int
CF_Queue_Front (CF_Queue_Ctx 	ctx,
				void			** element)
{
	int				result = 0;
	CF_List_Ctx		list = (CF_List_Ctx) ctx;
	CF_Traverser	traverser = NULL;

	result = CF_List_Front (list, &traverser);
	if (result < 0)
		return result;

	return CF_List_GetElement (traverser, element);
}

/**
 * 큐에 등록된 항목의 수를 가져옴
 *
 * @return 성공 시, 항목 수; 실패 시, 오류 코드
 *
 * @param ctx 큐 컨텍스트
 */
int
CF_Queue_GetSize (CF_Queue_Ctx ctx)
{
	return CF_List_GetSize ((CF_List_Ctx) ctx);
}
